@@ -17,38 +17,20 @@
 #include "ntp_string.h"
 #include "ntp_syslog.h"
 
-
-/*
- * #define away gcc __attribute__ if unavailable.
- */
-#ifndef __attribute__
-  /* This feature is available in gcc versions 2.5 and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || (defined(__STRICT_ANSI__))
-#  define __attribute__(Spec) /* empty */
-# endif
-  /*
-   * The __-protected variants of `format' and `printf' attributes are
-   * accepted by gcc versions 2.6.4 (effectively 2.7) and later.
-   */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __format__	format
-#  define __printf__	printf
-#  define __noreturn__	noreturn
-# endif
+#ifdef __GNUC__
+#define NTP_PRINTF(fmt, args) __attribute__((__format__(__printf__, fmt, args)))
+#else
+#define NTP_PRINTF(fmt, args)
 #endif
 
-extern	int	mprintf(const char *, ...)
-			__attribute__((__format__(__printf__, 1, 2)));
-extern	int	mfprintf(FILE *, const char *, ...)
-			__attribute__((__format__(__printf__, 2, 3)));
-extern	int	mvfprintf(FILE *, const char *, va_list)
-			__attribute__((__format__(__printf__, 2, 0)));
+extern	int	mprintf(const char *, ...) NTP_PRINTF(1, 2);
+extern	int	mfprintf(FILE *, const char *, ...) NTP_PRINTF(2, 3);
+extern	int	mvfprintf(FILE *, const char *, va_list) NTP_PRINTF(2, 0);
 extern	int	mvsnprintf(char *, size_t, const char *, va_list)
-			__attribute__((__format__(__printf__, 3, 0)));
+			NTP_PRINTF(3, 0);
 extern	int	msnprintf(char *, size_t, const char *, ...)
-			__attribute__((__format__(__printf__, 3, 4)));
-extern	void	msyslog(int, const char *, ...)
-			__attribute__((__format__(__printf__, 2, 3)));
+			NTP_PRINTF(3, 4);
+extern	void	msyslog(int, const char *, ...) NTP_PRINTF(2, 3);
 extern	void	init_logging	(const char *, u_int32, int);
 extern	int	change_logfile	(const char *, int);
 extern	void	setup_logfile	(const char *);
@@ -107,20 +89,21 @@ extern	int	ntp_getopt	(int, char **, const char *);
 extern	void	init_auth	(void);
 extern	void	init_lib	(void);
 extern	struct savekey *auth_findkey (keyid_t);
-extern	int	auth_moremem	(void);
+extern	void	auth_moremem	(int);
+extern	void	auth_prealloc_symkeys(int);
 extern	int	ymd2yd		(int, int, int);
 
 /* a_md5encrypt.c */
 extern	int	MD5authdecrypt	(int, u_char *, u_int32 *, int, int);
 extern	int	MD5authencrypt	(int, u_char *, u_int32 *, int);
-extern	void	MD5auth_setkey	(keyid_t, int, const u_char *, const int);
+extern	void	MD5auth_setkey	(keyid_t, int, const u_char *, size_t);
 extern	u_int32	addr2refid	(sockaddr_u *);
 
 /* emalloc.c */
 #ifndef EREALLOC_CALLSITE	/* ntp_malloc.h defines */
 extern	void *	ereallocz	(void *, size_t, size_t, int);
 #define	erealloczsite(p, n, o, z, f, l) ereallocz(p, n, o, (z))
-extern	void *	emalloc		(size_t);
+#define	emalloc(n)		ereallocz(NULL, n, 0, FALSE)
 #define	emalloc_zero(c)		ereallocz(NULL, (c), 0, TRUE)
 #define	erealloc(p, c)		ereallocz(p, (c), 0, FALSE)
 #define erealloc_zero(p, n, o)	ereallocz(p, n, (o), TRUE)
@@ -146,14 +129,15 @@ extern	char *	estrdup_impl	(const char *, const char *, int);
 extern	int	atoint		(const char *, long *);
 extern	int	atouint		(const char *, u_long *);
 extern	int	hextoint	(const char *, u_long *);
-extern	char *	humanlogtime	(void);
-extern	char *	humantime	(time_t);
-extern	char *	inttoa		(long);
-extern	char *	mfptoa		(u_long, u_long, short);
-extern	char *	mfptoms		(u_long, u_long, short);
-extern	const char * modetoa	(int);
+extern	const char *	humanlogtime	(void);
+extern	const char *	humantime	(time_t);
+extern	char *	mfptoa		(u_int32, u_int32, short);
+extern	char *	mfptoms		(u_int32, u_int32, short);
+extern	const char * modetoa	(size_t);
 extern	const char * eventstr	(int);
 extern	const char * ceventstr	(int);
+extern	const char * res_match_flags(u_short);
+extern	const char * res_access_flags(u_short);
 #ifdef KERNEL_PLL
 extern	const char * k_st_flags	(u_int32);
 #endif
@@ -164,13 +148,12 @@ extern	char *	numtohost	(u_int32);
 extern	const char * socktoa	(const sockaddr_u *);
 extern	const char * sockporttoa(const sockaddr_u *);
 extern	u_short	sock_hash	(const sockaddr_u *);
+extern	int	sockaddr_masktoprefixlen(const sockaddr_u *);
 extern	const char * socktohost	(const sockaddr_u *);
 extern	int	octtoint	(const char *, u_long *);
 extern	u_long	ranp2		(int);
 extern	const char *refnumtoa	(sockaddr_u *);
 extern	const char *refid_str	(u_int32, int);
-extern	int	tsftomsu	(u_long, int);
-extern	char *	uinttoa		(u_long);
 
 extern	int	decodenetnum	(const char *, sockaddr_u *);
 
@@ -202,9 +185,10 @@ extern int	authnumfreekeys;
  * The key cache. We cache the last key we looked at here.
  */
 extern keyid_t	cache_keyid;		/* key identifier */
-extern u_char *	cache_key;		/* key pointer */
 extern int	cache_type;		/* key type */
-extern u_int	cache_keylen;		/* key length */
+extern u_char *	cache_secret;		/* secret */
+extern u_short	cache_secretsize;	/* secret octets */
+extern u_short	cache_flags;		/* KEY_ bit flags */
 
 /* getopt.c */
 extern char *	ntp_optarg;		/* global argument pointer */
@@ -272,7 +256,10 @@ extern char *	ntp_strerror	(int e);
 #endif
 
 /* systime.c */
-extern double	sys_tick;		/* adjtime() resolution */
+extern double	sys_tick;		/* tick size or time to read */
+extern double	measured_tick;		/* non-overridable sys_tick */
+extern double	sys_fuzz;		/* min clock read latency */
+extern int	trunc_os_clock;		/* sys_tick > measured_tick */
 
 /* version.c */
 extern const char *Version;		/* version declaration */
