@@ -18,6 +18,7 @@
 #include "isc/error.h"
 #include "isc/strerror.h"
 #include "isc/formatcheck.h"
+#include "iosignal.h"
 
 #ifdef SIM
 # include "ntpsim.h"
@@ -77,6 +78,7 @@
 # include <apollo/base.h>
 #endif /* SYS_DOMAINOS */
 
+
 #include "recvbuff.h"
 #include "ntp_cmdargs.h"
 
@@ -105,20 +107,34 @@
 #ifdef HAVE_LINUX_CAPABILITIES
 # include <sys/capability.h>
 # include <sys/prctl.h>
-#endif
-#endif
+#endif /* HAVE_LINUX_CAPABILITIES */
+#if defined(HAVE_PRIV_H) && defined(HAVE_SOLARIS_PRIVS)
+# include <priv.h>
+#endif /* HAVE_PRIV_H */
+#endif /* HAVE_DROPROOT */
+
+#if defined (LIBSECCOMP) && (KERN_SECCOMP)
+/* # include <sys/types.h> */
+# include <sys/resource.h>
+# include <seccomp.h>
+#endif /* LIBSECCOMP and KERN_SECCOMP */
 
 #ifdef HAVE_DNSREGISTRATION
-#include <dns_sd.h>
+# include <dns_sd.h>
 DNSServiceRef mdns;
 #endif
 
 #ifdef HAVE_SETPGRP_0
-#define ntp_setpgrp(x, y)	setpgrp()
+# define ntp_setpgrp(x, y)	setpgrp()
 #else
-#define ntp_setpgrp(x, y)	setpgrp(x, y)
+# define ntp_setpgrp(x, y)	setpgrp(x, y)
 #endif
 
+#ifdef HAVE_SOLARIS_PRIVS
+# define LOWPRIVS "basic,sys_time,net_privaddr,proc_setid,!proc_info,!proc_session,!proc_exec"
+static priv_set_t *lowprivs = NULL;
+static priv_set_t *highprivs = NULL;
+#endif /* HAVE_SOLARIS_PRIVS */
 /*
  * Scheduling priority we run at
  */
@@ -152,8 +168,8 @@ int root_dropped;
 char *user;		/* User to switch to */
 char *group;		/* group to switch to */
 const char *chrootdir;	/* directory to chroot to */
-int sw_uid;
-int sw_gid;
+uid_t sw_uid;
+gid_t sw_gid;
 char *endp;
 struct group *gr;
 struct passwd *pw;
@@ -449,6 +465,9 @@ ntpdmain(
 	parse_cmdline_opts(&argc, &argv);
 # ifdef DEBUG
 	debug = OPT_VALUE_SET_DEBUG_LEVEL;
+#  ifdef HAVE_SETLINEBUF
+	setlinebuf(stdout);
+#  endif
 # endif
 
 	if (HAVE_OPT(NOFORK) || HAVE_OPT(QUIT)
@@ -487,7 +506,7 @@ ntpdmain(
 				" %s", saved_argv[i]);
 			cp += strlen(cp);
 		}
-		msyslog(LOG_NOTICE, "%s", buf);
+		msyslog(LOG_INFO, "%s", buf);
 	}
 
 	/*
@@ -682,6 +701,7 @@ ntpdmain(
 	}
 # endif
 
+	/* Setup stack size in preparation for locking pages in memory. */
 # if defined(HAVE_MLOCKALL)
 #  ifdef HAVE_SETRLIMIT
 	ntp_rlimit(RLIMIT_STACK, DFLT_RLIMIT_STACK * 4096, 4096, "4k");
@@ -695,12 +715,6 @@ ntpdmain(
 	ntp_rlimit(RLIMIT_MEMLOCK, DFLT_RLIMIT_MEMLOCK * 1024 * 1024, 1024 * 1024, "MB");
 #   endif	/* RLIMIT_MEMLOCK */
 #  endif	/* HAVE_SETRLIMIT */
-	/*
-	 * lock the process into memory
-	 */
-	if (!HAVE_OPT(SAVECONFIGQUIT) &&
-	    0 != mlockall(MCL_CURRENT|MCL_FUTURE))
-		msyslog(LOG_ERR, "mlockall(): %m");
 # else	/* !HAVE_MLOCKALL follows */
 #  ifdef HAVE_PLOCK
 #   ifdef PROCLOCK
@@ -713,22 +727,7 @@ ntpdmain(
 		msyslog(LOG_ERR,
 			"Cannot adjust stack limit for plock: %m");
 #    endif	/* _AIX */
-	/*
-	 * lock the process into memory
-	 */
-	if (!HAVE_OPT(SAVECONFIGQUIT) && 0 != plock(PROCLOCK))
-		msyslog(LOG_ERR, "plock(PROCLOCK): %m");
-#   else	/* !PROCLOCK follows  */
-#    ifdef TXTLOCK
-	/*
-	 * Lock text into ram
-	 */
-	if (!HAVE_OPT(SAVECONFIGQUIT) && 0 != plock(TXTLOCK))
-		msyslog(LOG_ERR, "plock(TXTLOCK) error: %m");
-#    else	/* !TXTLOCK follows */
-	msyslog(LOG_ERR, "plock() - don't know what to lock!");
-#    endif	/* !TXTLOCK */
-#   endif	/* !PROCLOCK */
+#   endif	/* PROCLOCK */
 #  endif	/* HAVE_PLOCK */
 # endif	/* !HAVE_MLOCKALL */
 
@@ -791,6 +790,38 @@ ntpdmain(
 	 * since this will definitely be different for the gizmo board.
 	 */
 	getconfig(argc, argv);
+
+	if (do_memlock) {
+# if defined(HAVE_MLOCKALL)
+		/*
+		 * lock the process into memory
+		 */
+		if (!HAVE_OPT(SAVECONFIGQUIT) &&
+		    0 != mlockall(MCL_CURRENT|MCL_FUTURE))
+			msyslog(LOG_ERR, "mlockall(): %m");
+# else	/* !HAVE_MLOCKALL follows */
+#  ifdef HAVE_PLOCK
+#   ifdef PROCLOCK
+		/*
+		 * lock the process into memory
+		 */
+		if (!HAVE_OPT(SAVECONFIGQUIT) && 0 != plock(PROCLOCK))
+			msyslog(LOG_ERR, "plock(PROCLOCK): %m");
+#   else	/* !PROCLOCK follows  */
+#    ifdef TXTLOCK
+		/*
+		 * Lock text into ram
+		 */
+		if (!HAVE_OPT(SAVECONFIGQUIT) && 0 != plock(TXTLOCK))
+			msyslog(LOG_ERR, "plock(TXTLOCK) error: %m");
+#    else	/* !TXTLOCK follows */
+		msyslog(LOG_ERR, "plock() - don't know what to lock!");
+#    endif	/* !TXTLOCK */
+#   endif	/* !PROCLOCK */
+#  endif	/* HAVE_PLOCK */
+# endif	/* !HAVE_MLOCKALL */
+	}
+
 	loop_config(LOOP_DRIFTINIT, 0);
 	report_event(EVNT_SYSRESTART, NULL, NULL);
 	initializing = FALSE;
@@ -805,13 +836,15 @@ ntpdmain(
 			msyslog( LOG_ERR, "prctl( PR_SET_KEEPCAPS, 1L ) failed: %m" );
 			exit(-1);
 		}
+#  elif HAVE_SOLARIS_PRIVS
+		/* Nothing to do here */
 #  else
 		/* we need a user to switch to */
 		if (user == NULL) {
 			msyslog(LOG_ERR, "Need user name to drop root privileges (see -u flag!)" );
 			exit(-1);
 		}
-#  endif	/* HAVE_LINUX_CAPABILITIES */
+#  endif	/* HAVE_LINUX_CAPABILITIES || HAVE_SOLARIS_PRIVS */
 
 		if (user != NULL) {
 			if (isdigit((unsigned char)*user)) {
@@ -876,6 +909,22 @@ getgroup:
 				exit (-1);
 			}
 		}
+#  ifdef HAVE_SOLARIS_PRIVS
+		if ((lowprivs = priv_str_to_set(LOWPRIVS, ",", NULL)) == NULL) {
+			msyslog(LOG_ERR, "priv_str_to_set() failed:%m");
+			exit(-1);
+		}
+		if ((highprivs = priv_allocset()) == NULL) {
+			msyslog(LOG_ERR, "priv_allocset() failed:%m");
+			exit(-1);
+		}
+		(void) getppriv(PRIV_PERMITTED, highprivs);
+		(void) priv_intersect(highprivs, lowprivs);
+		if (setppriv(PRIV_SET, PRIV_PERMITTED, lowprivs) == -1) {
+			msyslog(LOG_ERR, "setppriv() failed:%m");
+			exit(-1);
+		}
+#  endif /* HAVE_SOLARIS_PRIVS */
 		if (user && initgroups(user, sw_gid)) {
 			msyslog(LOG_ERR, "Cannot initgroups() to user `%s': %m", user);
 			exit (-1);
@@ -888,6 +937,10 @@ getgroup:
 			msyslog(LOG_ERR, "Cannot setegid() to group `%s': %m", group);
 			exit (-1);
 		}
+		if (group)
+			setgroups(1, &sw_gid);
+		else
+			initgroups(pw->pw_name, pw->pw_gid);
 		if (user && setuid(sw_uid)) {
 			msyslog(LOG_ERR, "Cannot setuid() to user `%s': %m", user);
 			exit (-1);
@@ -897,14 +950,14 @@ getgroup:
 			exit (-1);
 		}
 
-#  ifndef HAVE_LINUX_CAPABILITIES
+#  if !defined(HAVE_LINUX_CAPABILITIES) && !defined(HAVE_SOLARIS_PRIVS)
 		/*
 		 * for now assume that the privilege to bind to privileged ports
 		 * is associated with running with uid 0 - should be refined on
 		 * ports that allow binding to NTP_PORT with uid != 0
 		 */
 		disable_dynamic_updates |= (sw_uid != 0);  /* also notifies routing message listener */
-#  endif
+#  endif /* !HAVE_LINUX_CAPABILITIES && !HAVE_SOLARIS_PRIVS */
 
 		if (disable_dynamic_updates && interface_interval) {
 			interface_interval = 0;
@@ -939,27 +992,127 @@ getgroup:
 			cap_free(caps);
 		}
 #  endif	/* HAVE_LINUX_CAPABILITIES */
+#  ifdef HAVE_SOLARIS_PRIVS
+		if (priv_delset(lowprivs, "proc_setid") == -1) {
+			msyslog(LOG_ERR, "priv_delset() failed:%m");
+			exit(-1);
+		}
+		if (setppriv(PRIV_SET, PRIV_PERMITTED, lowprivs) == -1) {
+			msyslog(LOG_ERR, "setppriv() failed:%m");
+			exit(-1);
+		}
+		priv_freeset(lowprivs);
+		priv_freeset(highprivs);
+#  endif /* HAVE_SOLARIS_PRIVS */
 		root_dropped = TRUE;
 		fork_deferred_worker();
 	}	/* if (droproot) */
 # endif	/* HAVE_DROPROOT */
 
-	/*
-	 * Use select() on all on all input fd's for unlimited
-	 * time.  select() will terminate on SIGALARM or on the
-	 * reception of input.	Using select() means we can't do
-	 * robust signal handling and we get a potential race
-	 * between checking for alarms and doing the select().
-	 * Mostly harmless, I think.
-	 */
-	/*
-	 * On VMS, I suspect that select() can't be interrupted
-	 * by a "signal" either, so I take the easy way out and
-	 * have select() time out after one second.
-	 * System clock updates really aren't time-critical,
-	 * and - lacking a hardware reference clock - I have
-	 * yet to learn about anything else that is.
-	 */
+/* libssecomp sandboxing */
+#if defined (LIBSECCOMP) && (KERN_SECCOMP)
+	scmp_filter_ctx ctx;
+
+	if ((ctx = seccomp_init(SCMP_ACT_KILL)) < 0)
+		msyslog(LOG_ERR, "%s: seccomp_init(SCMP_ACT_KILL) failed: %m", __func__);
+	else {
+		msyslog(LOG_DEBUG, "%s: seccomp_init(SCMP_ACT_KILL) succeeded", __func__);
+	}
+
+#ifdef __x86_64__
+int scmp_sc[] = {
+	SCMP_SYS(adjtimex),
+	SCMP_SYS(bind),
+	SCMP_SYS(brk),
+	SCMP_SYS(chdir),
+	SCMP_SYS(clock_gettime),
+	SCMP_SYS(clock_settime),
+	SCMP_SYS(close),
+	SCMP_SYS(connect),
+	SCMP_SYS(exit_group),
+	SCMP_SYS(fstat),
+	SCMP_SYS(fsync),
+	SCMP_SYS(futex),
+	SCMP_SYS(getitimer),
+	SCMP_SYS(getsockname),
+	SCMP_SYS(ioctl),
+	SCMP_SYS(lseek),
+	SCMP_SYS(madvise),
+	SCMP_SYS(mmap),
+	SCMP_SYS(munmap),
+	SCMP_SYS(open),
+	SCMP_SYS(poll),
+	SCMP_SYS(read),
+	SCMP_SYS(recvmsg),
+	SCMP_SYS(rename),
+	SCMP_SYS(rt_sigaction),
+	SCMP_SYS(rt_sigprocmask),
+	SCMP_SYS(rt_sigreturn),
+	SCMP_SYS(select),
+	SCMP_SYS(sendto),
+	SCMP_SYS(setitimer),
+	SCMP_SYS(setsid),
+	SCMP_SYS(socket),
+	SCMP_SYS(stat),
+	SCMP_SYS(time),
+	SCMP_SYS(write),
+};
+#endif
+#ifdef __i386__
+int scmp_sc[] = {
+	SCMP_SYS(_newselect),
+	SCMP_SYS(adjtimex),
+	SCMP_SYS(brk),
+	SCMP_SYS(chdir),
+	SCMP_SYS(clock_gettime),
+	SCMP_SYS(clock_settime),
+	SCMP_SYS(close),
+	SCMP_SYS(exit_group),
+	SCMP_SYS(fsync),
+	SCMP_SYS(futex),
+	SCMP_SYS(getitimer),
+	SCMP_SYS(madvise),
+	SCMP_SYS(mmap),
+	SCMP_SYS(mmap2),
+	SCMP_SYS(munmap),
+	SCMP_SYS(open),
+	SCMP_SYS(poll),
+	SCMP_SYS(read),
+	SCMP_SYS(rename),
+	SCMP_SYS(rt_sigaction),
+	SCMP_SYS(rt_sigprocmask),
+	SCMP_SYS(select),
+	SCMP_SYS(setitimer),
+	SCMP_SYS(setsid),
+	SCMP_SYS(sigprocmask),
+	SCMP_SYS(sigreturn),
+	SCMP_SYS(socketcall),
+	SCMP_SYS(stat64),
+	SCMP_SYS(time),
+	SCMP_SYS(write),
+};
+#endif
+	{
+		int i;
+
+		for (i = 0; i < COUNTOF(scmp_sc); i++) {
+			if (seccomp_rule_add(ctx,
+			    SCMP_ACT_ALLOW, scmp_sc[i], 0) < 0) {
+				msyslog(LOG_ERR,
+				    "%s: seccomp_rule_add() failed: %m",
+				    __func__);
+			}
+		}
+	}
+
+	if (seccomp_load(ctx) < 0)
+		msyslog(LOG_ERR, "%s: seccomp_load() failed: %m",
+		    __func__);	
+	else {
+		msyslog(LOG_DEBUG, "%s: seccomp_load() succeeded", __func__);
+	}
+#endif /* LIBSECCOMP and KERN_SECCOMP */
+
 # ifdef HAVE_IO_COMPLETION_PORT
 
 	for (;;) {
@@ -968,12 +1121,8 @@ getgroup:
 
 	BLOCK_IO_AND_ALARM();
 	was_alarmed = FALSE;
-	for (;;) {
-#  ifndef HAVE_SIGNALED_IO
-		fd_set rdfdes;
-		int nfound;
-#  endif
 
+	for (;;) {
 		if (alarm_flag) {	/* alarmed? */
 			was_alarmed = TRUE;
 			alarm_flag = FALSE;
@@ -983,47 +1132,12 @@ getgroup:
 			/*
 			 * Nothing to do.  Wait for something.
 			 */
-#  ifndef HAVE_SIGNALED_IO
-			rdfdes = activefds;
-#   if !defined(VMS) && !defined(SYS_VXWORKS)
-			nfound = select(maxactivefd + 1, &rdfdes, NULL,
-					NULL, NULL);
-#   else	/* VMS, VxWorks */
-			/* make select() wake up after one second */
-			{
-				struct timeval t1;
+			io_handler();
+		}
 
-				t1.tv_sec = 1;
-				t1.tv_usec = 0;
-				nfound = select(maxactivefd + 1,
-						&rdfdes, NULL, NULL,
-						&t1);
-			}
-#   endif	/* VMS, VxWorks */
-			if (nfound > 0) {
-				l_fp ts;
-
-				get_systime(&ts);
-
-				input_handler(&ts);
-			} else if (nfound == -1 && errno != EINTR) {
-				msyslog(LOG_ERR, "select() error: %m");
-			}
-#   ifdef DEBUG
-			  else if (debug > 4) {
-				msyslog(LOG_DEBUG, "select(): nfound=%d, error: %m", nfound);
-			} else {
-				DPRINTF(1, ("select() returned %d: %m\n", nfound));
-			}
-#   endif /* DEBUG */
-#  else /* HAVE_SIGNALED_IO */
-
-			wait_for_signal();
-#  endif /* HAVE_SIGNALED_IO */
-			if (alarm_flag) {	/* alarmed? */
-				was_alarmed = TRUE;
-				alarm_flag = FALSE;
-			}
+		if (alarm_flag) {	/* alarmed? */
+			was_alarmed = TRUE;
+			alarm_flag = FALSE;
 		}
 
 		if (was_alarmed) {
@@ -1143,6 +1257,7 @@ finish(
 		sig_desc = "";
 	msyslog(LOG_NOTICE, "%s exiting on signal %d (%s)", progname,
 		sig, sig_desc);
+	/* See Bug 2513 and Bug 2522 re the unlink of PIDFILE */
 # ifdef HAVE_DNSREGISTRATION
 	if (mdns != NULL)
 		DNSServiceRefDeallocate(mdns);
