@@ -14,7 +14,7 @@
 #include "ntp_fp.h"
 #include "ntp.h"
 #include "ntp_io.h"
-#include "ntp_unixtime.h"
+#include "timevalops.h"
 #include "ntpdate.h"
 #include "ntp_string.h"
 #include "ntp_syslog.h"
@@ -109,19 +109,14 @@ static timer_t ntpdate_timerid;
  */
 s_char	sys_precision;		/* local clock precision (log2 s) */
 
-/*
- * Debugging flag
- */
-volatile int debug = 0;
-
 /*
  * File descriptor masks etc. for call to select
  */
 
 int ai_fam_templ;
-int nbsock;             /* the number of sockets used */
+int nbsock;			/* the number of sockets used */
 SOCKET fd[MAX_AF];
-int fd_family[MAX_AF];	/* to remember the socket family */
+int fd_family[MAX_AF];		/* to remember the socket family */
 #ifdef HAVE_POLL_H
 struct pollfd fdmask[MAX_AF];
 #else
@@ -329,7 +324,7 @@ ntpdatemain (
 	key_file = key_file_storage;
 
 	if (!ExpandEnvironmentStrings(KEYFILE, key_file, MAX_PATH))
-		msyslog(LOG_ERR, "ExpandEnvironmentStrings(KEYFILE) failed: %m\n");
+		msyslog(LOG_ERR, "ExpandEnvironmentStrings(KEYFILE) failed: %m");
 
 	ssl_applink();
 #endif /* SYS_WINNT */
@@ -756,7 +751,7 @@ receive(
 	 */
 	if (rbufp->recv_length == LEN_PKT_NOMAC)
 		has_mac = 0;
-	else if (rbufp->recv_length >= LEN_PKT_NOMAC)
+	else if (rbufp->recv_length >= (int)LEN_PKT_NOMAC)
 		has_mac = 1;
 	else {
 		if (debug)
@@ -833,7 +828,7 @@ receive(
 	if (LEAP_NOTINSYNC == PKT_LEAP(rpkt->li_vn_mode) &&
 	    STRATUM_PKT_UNSPEC == rpkt->stratum &&
 	    !memcmp("RATE", &rpkt->refid, 4)) {
-		msyslog(LOG_ERR, "%s rate limit response from server.\n",
+		msyslog(LOG_ERR, "%s rate limit response from server.",
 			stoa(&rbufp->recv_srcadr));
 		server->event_time = 0;
 		complete_servers++;
@@ -1357,7 +1352,7 @@ addserver(
 	char service[5];
 	sockaddr_u addr;
 
-	strncpy(service, "ntp", sizeof(service));
+	strlcpy(service, "ntp", sizeof(service));
 
 	/* Get host address. Looking for UDP datagram connection. */
 	ZERO(hints);
@@ -1378,13 +1373,13 @@ addserver(
 			   by waiting for resolution of several servers */
 			fprintf(stderr, "Exiting, name server cannot be used: %s (%d)",
 				gai_strerror(error), error);
-			msyslog(LOG_ERR, "name server cannot be used: %s (%d)\n",
+			msyslog(LOG_ERR, "name server cannot be used: %s (%d)",
 				gai_strerror(error), error);
 			exit(1);
 		}
 		fprintf(stderr, "Error resolving %s: %s (%d)\n", serv,
 			gai_strerror(error), error);
-		msyslog(LOG_ERR, "Can't find host %s: %s (%d)\n", serv,
+		msyslog(LOG_ERR, "Can't find host %s: %s (%d)", serv,
 			gai_strerror(error), error);
 		return;
 	}
@@ -1694,7 +1689,7 @@ init_io(void)
 	 * Open the socket
 	 */
 
-	strncpy(service, "ntp", sizeof(service));
+	strlcpy(service, "ntp", sizeof(service));
 
 	/*
 	 * Init hints addrinfo structure
@@ -2043,7 +2038,7 @@ l_adj_systime(
 	if (adjtv.tv_usec != 0 && !debug) {
 		if (adjtime(&adjtv, &oadjtv) < 0) {
 			msyslog(LOG_ERR, "Can't adjust the time of day: %m");
-			return 0;
+			exit(1);
 		}
 	}
 	return 1;
@@ -2187,57 +2182,6 @@ printserver(
 			   lfptoa(&pp->offset, 6));
 }
 
-#if !defined(HAVE_VSPRINTF)
-int
-vsprintf(
-	char *str,
-	const char *fmt,
-	va_list ap
-	)
-{
-	FILE f;
-	int len;
-
-	f._flag = _IOWRT+_IOSTRG;
-	f._ptr = str;
-	f._cnt = 32767;
-	len = _doprnt(fmt, ap, &f);
-	*f._ptr = 0;
-	return (len);
-}
-#endif
-
-#if 0
-/* override function in library since SA_RESTART makes ALL syscalls restart */
-#ifdef SA_RESTART
-void
-signal_no_reset(
-	int sig,
-	void (*func)()
-	)
-{
-	int n;
-	struct sigaction vec;
-
-	vec.sa_handler = func;
-	sigemptyset(&vec.sa_mask);
-	vec.sa_flags = 0;
-
-	while (1)
-	{
-		n = sigaction(sig, &vec, NULL);
-		if (n == -1 && errno == EINTR)
-			continue;
-		break;
-	}
-	if (n == -1)
-	{
-		perror("sigaction");
-		exit(1);
-	}
-}
-#endif
-#endif
 
 #ifdef HAVE_NETINFO
 static ni_namelist *
