@@ -9,7 +9,7 @@
 /*
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (C) 1992-2014 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -62,14 +62,14 @@ cook_xml_text(char * pzData);
 static char *
 handle_struct(tOptions * opts, tOptState * ost, char * txt, int dir);
 
-static char *
-parse_keyword(tOptions * opts, char * txt, tOptionValue * typ);
+static char const *
+parse_keyword(tOptions * opts, char const * txt, tOptionValue * typ);
 
-static char *
-parse_set_mem(tOptions * opts, char * txt, tOptionValue * typ);
+static char const *
+parse_set_mem(tOptions * opts, char const * txt, tOptionValue * typ);
 
-static char *
-parse_value(char * txt, tOptionValue * typ);
+static char const *
+parse_value(char const * txt, tOptionValue * typ);
 /* = = = END-STATIC-FORWARD = = = */
 
 /**
@@ -77,8 +77,8 @@ parse_value(char * txt, tOptionValue * typ);
  *  @param[in] txt   start of skpped text
  *  @returns   character after skipped text
  */
-inline static char *
-skip_unkn(char * txt)
+inline static char const *
+skip_unkn(char const * txt)
 {
     txt = BRK_END_XML_TOKEN_CHARS(txt);
     return (*txt == NUL) ? NULL : txt;
@@ -184,7 +184,7 @@ optionFindValue(const tOptDesc * odesc, char const * name, char const * val)
     else do {
         tArgList* argl  = odesc->optCookie;
         int       argct = argl->useCt;
-        void **   poptv = (void**)(argl->apzArgs);
+        void **   poptv = (void**)(intptr_t)(argl->apzArgs);
 
         if (argct == 0) {
             errno = ENOENT;
@@ -266,7 +266,7 @@ optionFindNextValue(const tOptDesc * odesc, const tOptionValue * pPrevVal,
     else do {
         tArgList* argl = odesc->optCookie;
         int    ct   = argl->useCt;
-        void** poptv = (void**)argl->apzArgs;
+        void** poptv = (void**)(intptr_t)argl->apzArgs;
 
         while (--ct >= 0) {
             tOptionValue* pOV = *(poptv++);
@@ -326,7 +326,7 @@ optionGetValue(tOptionValue const * oov, char const * vname)
 
     if (arg_list->useCt > 0) {
         int     ct     = arg_list->useCt;
-        void ** ovlist = (void**)(arg_list->apzArgs);
+        void ** ovlist = (void**)(intptr_t)(arg_list->apzArgs);
 
         if (vname == NULL) {
             res = (tOptionValue*)*ovlist;
@@ -385,7 +385,7 @@ optionNextValue(tOptionValue const * ov_list,tOptionValue const * oov )
     arg_list = ov_list->v.nestVal;
     {
         int     ct    = arg_list->useCt;
-        void ** o_list = (void**)(arg_list->apzArgs);
+        void ** o_list = (void**)(intptr_t)(arg_list->apzArgs);
 
         while (ct-- > 0) {
             tOptionValue * nov = *(o_list++);
@@ -415,12 +415,18 @@ file_preset(tOptions * opts, char const * fname, int dir)
     tmap_info_t       cfgfile;
     tOptState         optst = OPTSTATE_INITIALIZER(PRESET);
     opt_state_mask_t  st_flags = optst.flags;
+    opt_state_mask_t  fl_save  = opts->fOptSet;
     char *            ftext =
         text_mmap(fname, PROT_READ|PROT_WRITE, MAP_PRIVATE, &cfgfile);
 
     if (TEXT_MMAP_FAILED_ADDR(ftext))
         return;
 
+    /*
+     * While processing config files, we ignore errors.
+     */
+    opts->fOptSet &= ~OPTPROC_ERRSTOP;
+
     if (dir == DIRECTION_CALLED) {
         st_flags = OPTST_DEFINED;
         dir   = DIRECTION_PROCESS;
@@ -483,6 +489,7 @@ file_preset(tOptions * opts, char const * fname, int dir)
 
  all_done:
     text_munmap(&cfgfile);
+    opts->fOptSet = fl_save;
 }
 
 /**
@@ -519,7 +526,7 @@ handle_cfg(tOptions * opts, tOptState * ost, char * txt, int dir)
     if (txt > pzEnd) {
     name_only:
         *pzEnd++ = NUL;
-        loadOptionLine(opts, ost, pzName, dir, OPTION_LOAD_UNCOOKED);
+        load_opt_line(opts, ost, pzName, dir, OPTION_LOAD_UNCOOKED);
         return pzEnd;
     }
 
@@ -575,7 +582,7 @@ handle_cfg(tOptions * opts, tOptState * ost, char * txt, int dir)
      *  "pzName" points to what looks like text for one option/configurable.
      *  It is NUL terminated.  Process it.
      */
-    loadOptionLine(opts, ost, pzName, dir, OPTION_LOAD_UNCOOKED);
+    load_opt_line(opts, ost, pzName, dir, OPTION_LOAD_UNCOOKED);
 
     return pzEnd;
 }
@@ -924,7 +931,8 @@ handle_struct(tOptions * opts, tOptState * ost, char * txt, int dir)
     switch (*txt) {
     case ' ':
     case '\t':
-        txt = parse_attrs(opts, SPN_WHITESPACE_CHARS(txt), &mode, &valu);
+        txt = (void *)(intptr_t)parse_attrs(
+            opts, SPN_WHITESPACE_CHARS(txt), &mode, &valu);
         if (txt == NULL)
             return txt;
         if (*txt == '>')
@@ -938,7 +946,7 @@ handle_struct(tOptions * opts, tOptState * ost, char * txt, int dir)
             return NULL;
         *txt = NUL;
         txt += 2;
-        loadOptionLine(opts, ost, pzName, dir, mode);
+        load_opt_line(opts, ost, pzName, dir, mode);
         return txt;
 
     case '>':
@@ -962,7 +970,7 @@ handle_struct(tOptions * opts, tOptState * ost, char * txt, int dir)
         return txt;
 
     /*
-     *  Rejoin the name and value for parsing by "loadOptionLine()".
+     *  Rejoin the name and value for parsing by "load_opt_line()".
      *  Erase any attributes parsed by "parse_attrs()".
      */
     memset(pcNulPoint, ' ', (size_t)(pzData - pcNulPoint));
@@ -979,7 +987,7 @@ handle_struct(tOptions * opts, tOptState * ost, char * txt, int dir)
      *  "pzName" points to what looks like text for one option/configurable.
      *  It is NUL terminated.  Process it.
      */
-    loadOptionLine(opts, ost, pzName, dir, mode);
+    load_opt_line(opts, ost, pzName, dir, mode);
 
     return txt;
 }
@@ -1130,7 +1138,7 @@ optionFileLoad(tOptions * opts, char const * prog)
      */
     {
         char const ** pp =
-            (char const **)(void *)&(opts->pzProgName);
+            (char const **)(void *)(intptr_t)&(opts->pzProgName);
         *pp = prog;
     }
 
@@ -1191,15 +1199,21 @@ optionLoadOpt(tOptions * opts, tOptDesc * odesc)
  *
  * @returns NULL on failure, otherwise the scan point
  */
-LOCAL char*
-parse_attrs(tOptions * opts, char * txt, tOptionLoadMode * pMode,
+LOCAL char const *
+parse_attrs(tOptions * opts, char const * txt, tOptionLoadMode * pMode,
             tOptionValue * pType)
 {
-    size_t len;
+    size_t len = 0;
 
     for (;;) {
         len = (size_t)(SPN_LOWER_CASE_CHARS(txt) - txt);
 
+        /*
+         * The enumeration used in this switch is derived from this switch
+         * statement itself.  The "find_option_xat_attribute_cmd" function
+         * will return XAT_CMD_MEMBERS for the "txt" string value
+         * "members", etc.
+         */
         switch (find_option_xat_attribute_cmd(txt, len)) {
         case XAT_CMD_TYPE:
             txt = parse_value(txt+len, pType);
@@ -1266,8 +1280,8 @@ parse_attrs(tOptions * opts, char * txt, tOptionLoadMode * pMode,
  *  @param     type  unused value type
  *  @returns   pointer after skipped text
  */
-static char *
-parse_keyword(tOptions * opts, char * txt, tOptionValue * typ)
+static char const *
+parse_keyword(tOptions * opts, char const * txt, tOptionValue * typ)
 {
     (void)opts;
     (void)typ;
@@ -1285,8 +1299,8 @@ parse_keyword(tOptions * opts, char * txt, tOptionValue * typ)
  *  @param     type  unused value type
  *  @returns   pointer after skipped text
  */
-static char *
-parse_set_mem(tOptions * opts, char * txt, tOptionValue * typ)
+static char const *
+parse_set_mem(tOptions * opts, char const * txt, tOptionValue * typ)
 {
     (void)opts;
     (void)typ;
@@ -1302,8 +1316,8 @@ parse_set_mem(tOptions * opts, char * txt, tOptionValue * typ)
  *  @param[out] typ  where to store the type found
  *  @returns    the next byte after the type name
  */
-static char *
-parse_value(char * txt, tOptionValue * typ)
+static char const *
+parse_value(char const * txt, tOptionValue * typ)
 {
     size_t len = 0;
 
@@ -1318,6 +1332,12 @@ parse_value(char * txt, tOptionValue * typ)
         return skip_unkn(txt + len);
     }
 
+    /*
+     * The enumeration used in this switch is derived from this switch
+     * statement itself.  The "find_option_value_type_cmd" function
+     * will return VTP_CMD_INTEGER for the "txt" string value
+     * "integer", etc.
+     */
     switch (find_option_value_type_cmd(txt, len)) {
     default:
     case VTP_INVALID_CMD: goto woops;
